/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "WTFLog2.h"

// Redirect to following function within Telescope scope.
extern void LogInfo( const char*, const char*, va_list );
extern void LogWarning( const char*, const char*, va_list );
extern void LogError( const char*, const char*, va_list );
extern void LogDebug( const char*, const char*, va_list );

unsigned int s_levelBitmask = 0;

void SwitchWTFLog2Level( unsigned int level, bool enabled )
{
	unsigned int mask = ( 1 << static_cast<int>( level ) );
	if( enabled )
		s_levelBitmask |= mask;
	else
		s_levelBitmask &= ~mask;
}

#ifdef __cplusplus
extern "C" {
#endif

void WTFLog2( const WTFLogChannel& channel, WTFLogLevel level, const char* format, ... )
{
	if ( ( ( 1 << static_cast<int>(level) ) & s_levelBitmask ) == 0 )
		return;

	if ( channel.state == WTFLogChannelState::Off )
		return;

	char channelLabel[128];
	snprintf( channelLabel, sizeof(channelLabel), "%s.%s", channel.subsystem, channel.name );

	va_list args;
	va_start( args, format );
	if ( level == WTFLogLevel::Always || level == WTFLogLevel::Info )
		LogInfo( channelLabel, format, args );
	else if ( level == WTFLogLevel::Warning )
		LogWarning( channelLabel, format, args );
	else if ( level == WTFLogLevel::Error )
		LogError( channelLabel, format, args );
	else if ( level == WTFLogLevel::Debug )
		LogDebug( channelLabel, format, args );
	va_end( args );
}

#ifdef __cplusplus
}
#endif
//-TELESCOPE_LIB_CHANGES
