/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once
#include <Universal/TelescopePlatformConfig.h>
#include <mutex>
#include <wtf/StdMap.h>
#include <wtf/Vector.h>
#include <wtf/text/StringHash.h>
#include <wtf/text/WTFString.h>


namespace Telescope
{
#if TELESCOPE( MEMORY_TRACING )
	class TelescopeMemoryTracing
	{
		enum EInitStatus
		{
			eIS_None,
			eIS_Initing,
			eIS_Inited,

			eIS_Max
		};

		enum ETracingStatus
		{
			eTS_Ready,			//	OK to trace next request
			eTS_Paused,			//	tracing, trace will cause recursive call allocate, prevent infinite recursive
			eTS_Stopped,		//	initialized, but feature is not enabled

			eTS_Max				//	uninitialized
		};


	public:
		static TelescopeMemoryTracing* GetInstance();
		static void ReleaseInstance();

		//	Probably be used in future, don't remove
		void PreUpdate() {}
		void PostUpdate() {}

		void EnableMemoryLeakBacktrace();
		void DisableMemoryLeakBacktrace();
		void DumpMemoryLeakBacktrace( bool bOutputSummary, int iGreaterThan );
		void ClearMemoryLeakBacktrace();

		void CheckBacktrace( void* result );
		void RemoveBacktrace( void* result );


	private:
		void Init();
		void Uninit();
		void PauseTracing();
		void ResumeTracing();
		void DumpMemoryLeakBacktraceInternal( bool bOutputSummary, int iGreaterThan );
		void TraceStack( void* result );
		void TraceStackInternal( WTF::Vector<void*>& rRlt );

		WTF::String ConvertStackAddrToString( const WTF::Vector<void*>& rCallstackAddr );
		unsigned long long HashCallStack( const WTF::Vector<void*>& rCS );

		bool m_bDebuggerAPIInited = false;
		EInitStatus m_eInitStatus = eIS_None;
		ETracingStatus m_eTracingStatus = eTS_Max;
		ETracingStatus m_eOriginTracingStatus = eTS_Max;
		bool m_bClearAllocatedMemMap = false;
		bool m_bDumpAllCallstack = false;

		WTF::StdMap<void*, WTF::Vector<void*>>* m_pMem = nullptr;
		std::recursive_mutex* m_pMemLocker = nullptr;
	};
#endif
}
