/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once
#include <Universal/TelescopePlatformConfig.h>

#if TELESCOPE(COMMAND_SYSTEM)

#include <wtf/StdUnorderedMap.h>
#include <functional>
#include <Memory/TelescopeMemoryTracing.h>
#include <assert.h>


namespace Telescope
{
	struct StrCmp
	{
		bool operator()( char const* a, char const* b ) const
		{
			return std::strcmp( a, b ) < 0;
		}
	};

	class TelescopeCMD
	{
	public:
		TelescopeCMD();

		void ExecuteCMD( const char* pCMD, const char* pParam );

	private:
		void MemoryTracing_EnableMemoryLeakBacktrace( const char* pParam );
		void MemoryTracing_DisableMemoryLeakBacktrace( const char* pParam );
		void MemoryTracing_DumpMemoryLeakBacktrace( const char* pParam );
		void MemoryTracing_ClearMemoryLeakBacktrace( const char* pParam );

		void RegisterCommand( const char* pCMD, std::function<void( const char* )> pFn );
		void RegisterAllCommands();

		WTF::StdMap<const char*, std::function<void( const char* )>, StrCmp> g_mCommandCB;
	};
};

#endif