/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once

#include "TelescopeJavascriptValue.h"
#include <cassert>

//	TODO: move to TelescopeImplTest
class JSBoundObjectImp : public Telescope::IJSBoundObject
{
public:
	int getWebID() { return m_webID; }
	void setWebID( int id ) { m_webID = id; }
private:
	int m_webID;
};

typedef JSBoundObjectImp* ( *JSBoundObjCreator )();
typedef void ( *JSBoundObjDestroyer )( void* p );

template<class T>
JSBoundObjectImp* WebJSBoundObjCreator()
{
	JSBoundObjectImp* obj = new T();
	return obj;
}

template <class T>
void WebJSBoundObjDestroyer( void* p )
{
	T* reinterpretedP = reinterpret_cast< T* >( p );
	assert( reinterpretedP );
	delete reinterpretedP;
}

enum
{
	JSAPI_FULL_SCREEN,
	JSAPI_ON_LOAD_COMPLETE,
	JSAPI_PRINT,
	JSAPI_COUNT
};

extern JSBoundObjCreator g_jsApiCreator[];
extern JSBoundObjDestroyer g_jsApiDestroyer[];