/*
 * Copyright (C) 2020 Activision Publishing, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials
 *    provided with the distribution.
 *
 * 3. Neither the name of Activision Publishing, Inc. nor the names of
 *    its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef TELESCOPELIB_INCLUDE
#define TELESCOPELIB_INCLUDE

#include "JSONInterfaces.h"

class CompressionWrapperBase;
namespace Telescope
{
	class IMemoryAllocator;
	class HttpWrapperBase;
	class HashingWrapperBase;
	class CryptoWrapperBase;
	class AssetLoaderWrapperBase;
	class LogWrapperBase;
	class SystemAPIWrapperBase;
	class ErrorHandlingWrapperBase;
	class WebView;

	constexpr const unsigned int LOG_TYPE_DEBUG =   ( 1 << 0 );
	constexpr const unsigned int LOG_TYPE_INFO =    ( 1 << 1 );
	constexpr const unsigned int LOG_TYPE_WARNING = ( 1 << 2 );
	constexpr const unsigned int LOG_TYPE_ERROR =   ( 1 << 3 );
	constexpr const unsigned int LOG_TYPE_FATAL =   ( 1 << 4 );
	constexpr const unsigned int LOG_TYPE_ALL = LOG_TYPE_DEBUG | LOG_TYPE_INFO | LOG_TYPE_WARNING | LOG_TYPE_ERROR | LOG_TYPE_FATAL;

	typedef unsigned int LogType;

	class TelescopeConfig
	{
	public:
		IMemoryAllocator* memAllocator;
		// TODO: looks like the naming conversion is not unified between IMemoryAllocator and HttpWrapperInterface.
		HttpWrapperBase* httpWrapper;
		// TODO: CompressionWrapperBase should be wrapped by namespace Telescope
		CompressionWrapperBase* compressionImpl;
		HashingWrapperBase* hashingImpl;
		CryptoWrapperBase* cryptoImpl;
		AssetLoaderWrapperBase* assetLoaderImpl;
		LogWrapperBase* logImpl;
		SystemAPIWrapperBase* sysAPIImpl;
		ErrorHandlingWrapperBase* errorHandlingImpl;

		TelescopeConfig()
			: memAllocator(nullptr)
			, httpWrapper(nullptr)
			, compressionImpl(nullptr)
			, hashingImpl(nullptr)
			, cryptoImpl(nullptr)
			, assetLoaderImpl(nullptr)
			, logImpl(nullptr)
			, sysAPIImpl(nullptr)
			, errorHandlingImpl(nullptr)
		{}
	};

	class TelescopeLib
	{
	public:
		virtual void PreInit( TelescopeConfig* config );
		virtual bool Init( TelescopeConfig* config );
		virtual void Shutdown();

		virtual void Tick();

		virtual WebView* CreateView();
		virtual void DestroyView( WebView* view );

		virtual bool InstallFont( const unsigned char* fontData, size_t dataSize );

		virtual void InitFileCache(const char *cacheDirectory, size_t storageSizeLimitInBytes);

		virtual void ExecuteCMD( const char* pCmd, const char* pParam );

		virtual unsigned int getTelescopeCL();

		virtual void LogTypeKillSwitch( LogType logType, bool enabled );
		virtual void LogChannelKillSwitch( const char* moduleName, const char* systemName, bool enabled );

		// JSON Factory method
		virtual JsonWrapper::ValuePtr ParseJson(const char* pJsonInput, size_t jsonLength = 0);
	};
}
#endif
